#ifndef CSPACEGROUPELEMENT_H
#define CSPACEGROUPELEMENT_H

#include <vector>
#include <boost/rational.hpp>

using boost::rational;
using std::vector;

typedef vector<rational<int> >          rationalVector;
typedef vector<vector<rational<int> > > rationalMatrix;

const unsigned LatticeDim = 6;
const unsigned ComplexLatticeDim = 3;

class CLatticeElement : private rationalVector{
public:
  ~CLatticeElement();
  CLatticeElement();
  CLatticeElement(const rationalVector &n_alpha);

  void                          operator=(const rationalVector &n_alpha);
  bool                          operator==(const CLatticeElement &Element2) const;
  bool                          operator!=(const CLatticeElement &Element2) const;
  void                          operator+=(const CLatticeElement &b);
  void                          operator-=(const CLatticeElement &b);
  void                          operator*=(const int &b);
  void                          operator*=(const rational<int> &b);
  rational<int>                 operator*(const rationalVector &ChargeOperator) const;
  CLatticeElement               operator*(const int &b) const;
  const rational<int>          &operator[](const unsigned &alpha) const;

  bool                          IsZero() const;
  bool                          Rotate(const rationalMatrix &TwistMatrix, CLatticeElement &result) const;
  bool                          Set_n_alpha(const unsigned &alpha, const rational<int> &n_alpha);
  const rationalVector         &Vector() const {return *this;};
};


class CSpaceGroupElement{
public:
  ~CSpaceGroupElement();
  CSpaceGroupElement();
  CSpaceGroupElement(unsigned k, unsigned l);
  CSpaceGroupElement(unsigned k, unsigned l, const rationalVector &n_alpha);

  bool                          operator==(const CSpaceGroupElement &Element2) const;
  bool                          operator!=(const CSpaceGroupElement &Element2) const;
  rational<int>                 operator*(const rationalVector &ChargeOperator) const;
  
  bool                          NoTwist() const;
  bool                          IsZero() const;

  bool                          Set_k(const unsigned &k);
  bool                          Set_l(const unsigned &l);
  bool                          SetLatticeElement(const CLatticeElement &LatticeElement);
  bool                          Set_n_alpha(const rationalVector &n_alpha);
  bool                          Set_n_alpha(const unsigned &alpha, const rational<int> &n_alpha);
  
  const unsigned               &Get_k() const {return this->k;};
  const unsigned               &Get_l() const {return this->l;};
  const CLatticeElement        &GetLatticeElement() const {return this->LatticeElement;};
  const rational<int>          &Get_n(const unsigned &alpha) const {return this->LatticeElement[alpha];};
  
private:
  unsigned                      k;
  unsigned                      l;
  CLatticeElement               LatticeElement;
};

#endif
